package com.localmatters.flexiq.collector.browser;

import static com.localmatters.flexiq.collector.CollectorConstants.*;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import javax.ejb.EJB;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import org.ajax4jsf.dnd.event.DropEvent;

/**
 *
 * @author Daniel Cavalcanti
 */
public class MessageDisplayBean {
    
    // **********************************************************************
    // Class constants
    // **********************************************************************
    
    private static final List<String> FORBIDDEN_HEADERS = new ArrayList<String>();
    
    static {
        FORBIDDEN_HEADERS.add(ENCRYPTION_ALGORITHM);
        FORBIDDEN_HEADERS.add(ENCRYPTION_KEY);
    }
    
    // **********************************************************************
    // Instance variables
    // **********************************************************************
    
    // EJB resources ********************************************************
    
    @EJB()
    private BrowserService browser;
    
    // UI properties ********************************************************
    
    // search parameters properties
    private String connectionUsername;
    private String connectionPassword;
    private String queueName;
    private String messageSelector;
    private int maxMessages = 10;
    private boolean displayAll = true;
    
    // message and headers properties
    
    private List<String> available = new ArrayList<String>();
    private List<String> selected = new ArrayList<String>();
    private List<Message> messages = new ArrayList<Message>();
    
    // **********************************************************************
    // Constructors
    // **********************************************************************
    
    /**
     * Creates a new instance of MessageDisplayBean.
     */
    public MessageDisplayBean() {
    }
    
    // **********************************************************************
    // Getter/Setter methods
    // **********************************************************************
    
    public String getConnectionUsername() {
        return connectionUsername;
    }
    
    public void setConnectionUsername(String connectionUsername) {
        this.connectionUsername = connectionUsername;
    }
    
    public String getConnectionPassword() {
        return connectionPassword;
    }
    
    public void setConnectionPassword(String connectionPassword) {
        this.connectionPassword = connectionPassword;
    }
    
    public String getQueueName() {
        return queueName;
    }
    
    public void setQueueName(String queueName) {
        this.queueName = queueName;
    }
    
    public String getMessageSelector() {
        return messageSelector;
    }
    
    public void setMessageSelector(String messageSelector) {
        this.messageSelector = messageSelector;
    }
    
    public int getMaxMessages() {
        return maxMessages;
    }
    
    public void setMaxMessages(int maxMessages) {
        this.maxMessages = maxMessages;
    }
    
    public boolean isDisplayAll() {
        return displayAll;
    }
    
    public void setDisplayAll(boolean displayAll) {
        this.displayAll = displayAll;
    }
    
    public List<String> getAvailable() {
        return available;
    }
    
    public boolean isAvailableEmpty() {
        return available.isEmpty();
    }
    
    public List<String> getSelected() {
        return selected;
    }
    
    public boolean isSelectedEmpty() {
        return selected.isEmpty();
    }
    
    // **********************************************************************
    // Action methods
    // **********************************************************************
    
    // Control actions ******************************************************
    
    public String listMessages() {
        
        selected.clear();
        available.clear();
        messages.clear();
        
        try {
            
            messages =
                displayAll
                ? browser.getMessages(queueName, connectionUsername, connectionPassword)
                : browser.getMessages(queueName, connectionUsername, connectionPassword, maxMessages);
            
            buildSelectedHeaders();
            buildMessagesDataModel();
            
        } catch (Throwable ex) {
            // TODO log
            ex.printStackTrace();
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(ex.getMessage(), ex.getMessage()));
        }
        
        return null;
        
    }
    
    public String refresh() {
        return null;
    }
    
    public String clear() {
        
        selected.clear();
        available.clear();
        messages.clear();
        buildMessagesDataModel();
        
        return null;
    }
    
    // Headers actions ******************************************************
    
    public void drop(DropEvent event) {
        
        String from = event.getDragType();
        String header = event.getDragValue().toString();
        
        if (from.equals("from-selected")) {
            selected.remove(header);
            available.add(header);
        } else if (from.equals("from-available")) {
            available.remove(header);
            selected.add(header);
        }
        
    }
    
    // **********************************************************************
    // Auxiliary methods
    // **********************************************************************
    
    private void buildSelectedHeaders()
    throws JMSException {
        for (Message message : messages) {
            for (Enumeration properties = message.getPropertyNames() ; properties.hasMoreElements() ; ) {
                String property = (String) properties.nextElement();
                if (!FORBIDDEN_HEADERS.contains(property) && !selected.contains(property))
                    selected.add(property);
            }
        }
    }
    
    private void buildMessagesDataModel() {
        
    }
    
}
