/*
 * Local Matters, Inc. Confidential
 * OCO Source Materials
 * (C) Local Matters, Inc. 2007. All Rights Reserved.
 */
package com.localmatters.flexiq.security;

import java.security.GeneralSecurityException;
import java.security.Key;
import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.SecretKeySpec;

/**
 * An Advanced Encryption Standard (AES) cipher service implementation.
 *
 * @author Daniel Cavalcanti
 */
public class AESCipherService
    extends CipherServiceBase
    implements CipherService {
    
    // **********************************************************************
    // Class constants
    // **********************************************************************
    
    /**
     * The string constant identity a AES key in JCE.
     */
    public static final String AES_KEY = "AES";
    
    /**
     * The standard algorithm name for AES.
     */
    public static final String AES_CIPHER = "AES";
    
    // **********************************************************************
    // Constructors
    // **********************************************************************
    
    /**
     * Creates a new instance of AESCipherService.
     *
     * @throws java.security.GeneralSecurityException If cannot create cipher service.
     */
    public AESCipherService()
    throws GeneralSecurityException {
        cipher = Cipher.getInstance(AES_CIPHER);
        keygen = KeyGenerator.getInstance(AES_KEY);
        setKey(keygen.generateKey());
    }
    
    /**
     * Creates a new instance of AESCipherService.
     * 
     * @param key The cipher key.
     * @throws java.security.GeneralSecurityException If cannot create cipher service.
     */
    public AESCipherService(Key key)
    throws GeneralSecurityException {
        cipher = Cipher.getInstance(AES_CIPHER);
        setKey(key);
    }
    
    // **********************************************************************
    // SecurityService methods
    // **********************************************************************
    
    public void importKey(byte[] key)
    throws GeneralSecurityException {
        this.key = new SecretKeySpec(key, AES_KEY);
    }
    
}
