/*
 * Local Matters, Inc. Confidential
 * OCO Source Materials
 * (C) Local Matters, Inc. 2007. All Rights Reserved.
 */
package com.localmatters.flexiq.security;

import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.Key;

/**
 * A general use cipher service.
 *
 * @author Daniel Cavalcanti
 */
public interface CipherService {
    
    /**
     * Gets the cipher key.
     *
     * @return The cipher key.
     */
    public Key getKey();
    
    /**
     * Sets the cipher key.
     * @param key The cipher key.
     * @throws java.security.InvalidKeyException If key is invalid for cipher.
     */
    public void setKey(Key key)
    throws InvalidKeyException;
    
    /**
     * Exports the cipher key.
     *
     * @throws java.security.GeneralSecurityException If cannot export cipher key.
     * @return The exported cipher key.
     */
    public byte[] exportKey()
    throws GeneralSecurityException;
    
    /**
     * Imports cipher key.
     *
     * @param key The cipher key.
     * @throws java.security.GeneralSecurityException If cannot import cipher key.
     */
    public void importKey(byte[] key)
    throws GeneralSecurityException;
    
    /**
     * Encrypts the cleartext into ciphertext.
     *
     * @param cleartext The cleartext to encrypt.
     * @throws java.lang.NullPointerException If cleartext is null.
     * @throws java.security.GeneralSecurityException If encryption fails.
     * @return The ciphertext.
     */
    public byte[] encrypt(byte[] cleartext)
    throws NullPointerException, GeneralSecurityException;
    
    /**
     * Decrypts the ciphertext into cleartext.
     *
     * @param ciphertext The ciphertext to decrypt.
     * @throws java.lang.NullPointerException If cuphertext is null.
     * @throws java.security.GeneralSecurityException If decryption fails.
     * @return The cleartext.
     */
    public byte[] decrypt(byte[] ciphertext)
    throws NullPointerException, GeneralSecurityException;
    
}
