/*
 * Local Matters, Inc. Confidential
 * OCO Source Materials
 * (C) Local Matters, Inc. 2007. All Rights Reserved.
 */
package com.localmatters.flexiq.security.store;

import java.util.List;

/**
 * The definition for a security store service.
 *
 * @author Daniel Cavalcanti
 */
public interface SecurityStoreService {
    
    /**
     * Returns a list of service names.
     *
     * @return A list of service names.
     */
    public List<String> getServiceNames();
    
    /**
     * Determines if the file store contains a service with <code>serviceName</code>.
     *
     * @param serviceName The service name.
     * @return true iff file store contains a service with the service name.
     */
    public boolean exist(String serviceName);
    
    /**
     * Creates a new service container with name <code>serviceName</code>.
     *
     * @param serviceName The service name.
     * @throws java.lang.NullPointerException If serviceName is null.
     * @throws java.lang.IllegalArgumentException If there is a service with serviceName already.
     */
    public void createService(String serviceName)
    throws NullPointerException, IllegalArgumentException;
    
    /**
     * Returns a list of usernames for a particular service.
     * 
     * @param serviceName The service name
     * @return The usersnames for the service.
     * @throws java.lang.NullPointerException If serviceName if null.
     * @throws java.lang.IllegalArgumentException If serviceName does not exist.
     */
    public List<String> getUsernames(String serviceName)
    throws NullPointerException, IllegalArgumentException;
    
    /**
     * Create a user for a particular service.
     *
     * @param serviceName The service name.
     * @param username The user's username.
     * @param password The user's password.
     * @throws java.lang.NullPointerException If any of the parameters is null.
     * @throws java.lang.IllegalArgumentException If there user already exists for that service.
     */
    public void createUser(String serviceName, String username, String password)
    throws NullPointerException, IllegalArgumentException;
    
    /**
     * Gets the (decrypted) password for a username of a particular service.
     * 
     * @param username The username.
     * @param serviceName The service name.
     * @return The user's (decrypted) password.
     * @throws java.lang.NullPointerException If any of the parameters is null.
     * @throws java.lang.IllegalArgumentException If there is no such service or user with that username for the specified service.
     */
    public String getPassword(String serviceName, String username)
    throws NullPointerException, IllegalArgumentException;
    
    /**
     * Sets the password for the username of a particular service.
     *
     * @param serviceName The service name.
     * @param username The user's username.
     * @param password The user's password.
     * @throws java.lang.NullPointerException If any of the parameters is null.
     * @throws java.lang.IllegalArgumentException If there is no such service or user with that username for the specified service.
     */
    public void setPassword(String serviceName, String username, String password)
    throws NullPointerException, IllegalArgumentException;
    
    /**
     * Saves any changes into the persistent layer.
     */
    public void persist();
    
    /**
     * Closes the file.
     */
    public void close();
    
}
